import { CallableObject } from '@s-libs/js-core';
import { Observable, Subscriber } from 'rxjs';
import { ChildStore, RootStore } from './index';
/** @hidden */
declare type GetSlice<T> = <K extends keyof T>(attr: K) => Store<T[K]>;
export interface Store<T> extends GetSlice<T> {
    /**
     * Select a slice of the store to operate on. For example `store('currentUser')` will return a new `Store` that represents the `currentUser` property.
     */
    <K extends keyof T, V extends T[K]>(attr: K): Store<V>;
}
export declare abstract class Store<T> extends CallableObject<GetSlice<T>> {
    getRootStore: () => RootStore<object>;
    /**
     * An `Observable` of the state of this store object.
     */
    readonly $: Observable<T>;
    protected subscribers: Map<Subscriber<T>, T | undefined>;
    protected activeChildren: Record<string, Set<Store<unknown>>>;
    protected lastKnownState?: T;
    private lastKnownStateChanged;
    constructor(getRootStore: () => RootStore<object>);
    /**
     * Retrieve the current state represented by this store object.
     */
    abstract state(): T;
    /**
     * Replace the state represented by this store object with the given value.
     */
    abstract set(value: T): void;
    /**
     * Removes the state represented by this store object from its parent. E.g. to remove the current user:
     *
     * ```ts
     * store('currentUser').delete();
     * ```
     */
    abstract delete(): void;
    /**
     * @returns whether the given `Store` operates on the same slice of the store as this object.
     */
    abstract refersToSameStateAs(other: Store<T>): boolean;
    /**
     * Assigns the given values to state of this store object. The resulting state will be like `Object.assign(store.state(), value)`.
     */
    assign(value: Partial<T>): void;
    /**
     * Runs `func` on the state and replaces it with the return value. The first argument to `func` will be the state, followed by the arguments in `args`.
     *
     * WARNING: You SHOULD NOT use a function that will mutate the state.
     */
    setUsing<A extends any[]>(func: (state: T, ...args: A) => T, ...args: A): void;
    /**
     * Runs `func` on a shallow clone of the state, replacing the state with the clone. The first argument to `func` will be the cloned state, followed by the arguments in `args`.
     *
     * WARNING: You SHOULD NOT use a function that will mutate nested objects within the state.
     */
    mutateUsing<A extends any[]>(func: (state: T, ...args: A) => void, ...args: A): void;
    protected abstract maybeActivate(): void;
    protected abstract maybeDeactivate(): void;
    protected updateState(value: any): void;
    protected maybeEmit(): void;
    protected isChildActive(parent: Store<any>, key: any, child: ChildStore<any>): boolean;
    protected activateChild(parent: Store<any>, key: any, child: ChildStore<any>): void;
    protected deactivateChild(parent: Store<any>, key: any, child: ChildStore<any>): void;
}
export {};
